/**
 * Copyright (c) 2015 Guyon Roche
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the 'Software'), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:</p>
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 * 
 */
'use strict';

// translate style data between xlsx <--> model

var Styles = module.exports = function(workbook) {
    this.workbook = workbook;
};

Styles.prototype = {
    addStyles: function(zip, model) {
        return utils.fetchTemplate('./xlsx/styles.xml')
            .then(function(template){
                var styleModel = {};
                
                // build styleModel from worksheets
                
                return template(styleModel);
            })
            .then(function(data) {
                zip.append(data, { name: '/xl/styles.xml' });
                return zip;
            });
    }
};

Styles.numFmts = [
    { id: 7, text: '""#,##0.00;\\-""#,##0.00', code: '&quot;&quot;#,##0.00;\-&quot;&quot;#,##0.00' },
    { id: 8, text: '""#,##0.00;[Red]\-""#,##0.00', code: '&quot;&quot;#,##0.00;[Red]\-&quot;&quot;#,##0.00' },
    { id: 44, text: '_-""* #,##0.00_-;\-""* #,##0.00_-;_-""* "-"??_-;_-@_-', code: '_-&quot;&quot;* #,##0.00_-;\-&quot;&quot;* #,##0.00_-;_-&quot;&quot;* &quot;-&quot;??_-;_-@_-' },
    { id: 164, text: '0.000', code: '0.000' },
    { id: 165, text: '0.00;[Red]0.00', code: '0.00;[Red]0.00' },
    { id: 166, text: '0.00_ ;\-0.00\ ', code: '0.00_ ;\-0.00\ ' },
    { id: 167, text: '0.00_ ;[Red]\-0.00\ ', code: '0.00_ ;[Red]\-0.00\ ' },
    { id: 168, text: '""#,##0.00', code: '&quot;&quot;#,##0.00' },
    { id: 169, text: '[$$-409]#,##0.00', code: '[$$-409]#,##0.00' },
    { id: 170, text: '[$-2]\ #,##0.00', code: '[$-2]\ #,##0.00' },
    { id: 171, text: '#,##0.00\ [$-1]', code: '#,##0.00\ [$-1]' },
    { id: 172, text: '""#,##0.000;[Red]""#,##0.000', code: '&quot;&quot;#,##0.000;[Red]&quot;&quot;#,##0.000' },
    { id: 173, text: '_-[$$-409]* #,##0.00_ ;_-[$$-409]* \-#,##0.00\ ;_-[$$-409]* "-"??_ ;_-@_ ', code: '_-[$$-409]* #,##0.00_ ;_-[$$-409]* \-#,##0.00\ ;_-[$$-409]* &quot;-&quot;??_ ;_-@_ ' },
    { id: 174, text: '_-""* #,##0.0000_-;\-""* #,##0.0000_-;_-""* "-"????_-;_-@_-', code: '_-&quot;&quot;* #,##0.0000_-;\-&quot;&quot;* #,##0.0000_-;_-&quot;&quot;* &quot;-&quot;????_-;_-@_-' },
    { id: 175, text: '_-[$-2]\ * #,##0.00_-;\-[$-2]\ * #,##0.00_-;_-[$-2]\ * "-"??_-;_-@_-', code: '_-[$-2]\ * #,##0.00_-;\-[$-2]\ * #,##0.00_-;_-[$-2]\ * &quot;-&quot;??_-;_-@_-' },
    { id: 176, text: '_-* #,##0.00\ [$-1]_-;\-* #,##0.00\ [$-1]_-;_-* "-"??\ [$-1]_-;_-@_-', code: '_-* #,##0.00\ [$-1]_-;\-* #,##0.00\ [$-1]_-;_-* &quot;-&quot;??\ [$-1]_-;_-@_-' },
    { id: 178, text: '[$-F800]dddd\,\ mmmm\ dd\,\ yyyy', code: '[$-F800]dddd\,\ mmmm\ dd\,\ yyyy' },
    { id: 179, text: 'dd/mm/yyyy;@', code: 'dd/mm/yyyy;@' },
    { id: 180, text: 'dd/mm/yy;@', code: 'dd/mm/yy;@' },
    { id: 181, text: 'd/m/yy;@', code: 'd/m/yy;@' },
    { id: 182, text: 'd\.m\.yy;@', code: 'd\.m\.yy;@' },
    { id: 183, text: 'yyyy\-mm\-dd;@', code: 'yyyy\-mm\-dd;@' },
    { id: 184, text: '[$-809]dd\ mmmm\ yyyy;@', code: '[$-809]dd\ mmmm\ yyyy;@' },
    { id: 185, text: '[$-809]d\ mmmm\ yyyy;@', code: '[$-809]d\ mmmm\ yyyy;@' },
    { id: 186, text: 'm/d;@', code: 'm/d;@' },
    { id: 187, text: 'm/d/yy;@', code: 'm/d/yy;@' },
    { id: 188, text: 'mm/dd/yy;@', code: 'mm/dd/yy;@' },
    { id: 189, text: '[$-409]d\-mmm;@', code: '[$-409]d\-mmm;@' },
    { id: 190, text: '[$-409]d\-mmm\-yy;@', code: '[$-409]d\-mmm\-yy;@' },
    { id: 191, text: '[$-409]dd\-mmm\-yy;@', code: '[$-409]dd\-mmm\-yy;@' },
    { id: 192, text: 'd/m;@', code: 'd/m;@' },
    { id: 193, text: '[$-40C]d\-mmm;@', code: '[$-40C]d\-mmm;@' },
    { id: 194, text: '[$-40C]d\-mmm\-yy;@', code: '[$-40C]d\-mmm\-yy;@' },
    { id: 195, text: '[$-40C]dd\-mmm\-yy;@', code: '[$-40C]dd\-mmm\-yy;@' },
    { id: 196, text: '[$-40C]mmm\-yy;@', code: '[$-40C]mmm\-yy;@' },
    { id: 197, text: '[$-40C]mmmm\-yy;@', code: '[$-40C]mmmm\-yy;@' },
    { id: 198, text: '[$-40C]d\ mmmm\ yyyy;@', code: '[$-40C]d\ mmmm\ yyyy;@' },
    { id: 199, text: '[$-409]d/m/yy\ h:mm\ AM/PM;@', code: '[$-409]d/m/yy\ h:mm\ AM/PM;@' },
    { id: 200, text: 'd/m/yy\ h:mm;@', code: 'd/m/yy\ h:mm;@' },
    { id: 201, text: '[$-40C]mmmmm;@', code: '[$-40C]mmmmm;@' },
    { id: 202, text: '[$-40C]mmmmm\-yy;@', code: '[$-40C]mmmmm\-yy;@' },
    { id: 203, text: 'm/d/yyyy;@', code: 'm/d/yyyy;@' },
    { id: 204, text: '[$-40C]d\-mmm\-yyyy;@', code: '[$-40C]d\-mmm\-yyyy;@' },
    { id: 206, text: '[$-F400]h:mm:ss\ AM/PM', code: '[$-F400]h:mm:ss\ AM/PM' },
    { id: 207, text: 'hh:mm:ss;@', code: 'hh:mm:ss;@' },
    { id: 208, text: 'h:mm:ss;@', code: 'h:mm:ss;@' },
    { id: 209, text: '[$-409]hh:mm:ss\ AM/PM;@', code: '[$-409]hh:mm:ss\ AM/PM;@' },
    { id: 210, text: '[$-409]h:mm:ss\ AM/PM;@', code: '[$-409]h:mm:ss\ AM/PM;@' },
    { id: 211, text: '[h]:mm:ss;@', code: '[h]:mm:ss;@' },
    { id: 212, text: 'h:mm;@', code: 'h:mm;@' },
    { id: 213, text: 'mm:ss.0;@', code: 'mm:ss.0;@' },
    { id: 214, text: '#\ ???/???', code: '#\ ???/???' },
    { id: 215, text: '#\ ?/2', code: '#\ ?/2' },
    { id: 216, text: '#\ ?/4', code: '#\ ?/4' },
    { id: 217, text: '#\ ?/8', code: '#\ ?/8' },
    { id: 218, text: '#\ ??/16', code: '#\ ??/16' },
    { id: 219, text: '#\ ?/10', code: '#\ ?/10' },
    { id: 220, text: '#\ ??/100', code: '#\ ??/100' },
    { id: 221, text: '0.0000E+00', code: '0.0000E+00' },
    { id: 222, text: '00000', code: '00000' },
    { id: 223, text: '00000\-0000', code: '00000\-0000' },
    { id: 224, text: '[&lt;=9999999]###\-####;\(###\)\ ###\-####', code: '[&lt;=9999999]###\-####;\(###\)\ ###\-####' },
    { id: 225, text: '000\-00\-0000', code: '000\-00\-0000' },
    { id: 226, text: '[&gt;=3000000000000]#" "##" "##" "##" "###" "###" | "##;#" "##" "##" "##" "###" "###', code: '[&gt;=3000000000000]#&quot; &quot;##&quot; &quot;##&quot; &quot;##&quot; &quot;###&quot; &quot;###&quot; | &quot;##;#&quot; &quot;##&quot; &quot;##&quot; &quot;##&quot; &quot;###&quot; &quot;###' },
    { id: 227, text: '0#" "##" "##" "##" "##', code: '0#&quot; &quot;##&quot; &quot;##&quot; &quot;##&quot; &quot;##' },
    { id: 228, text: '0##\-000" "00" "00', code: '0##\-000&quot; &quot;00&quot; &quot;00' },
    { id: 229, text: '[&gt;=10000000000]#\-###\-###\-###;[&gt;=10000000]\(###\)" "###\-####;000\-0000', code: '[&gt;=10000000000]#\-###\-###\-###;[&gt;=10000000]\(###\)&quot; &quot;###\-####;000\-0000' },
    { id: 230, text: '##" "00" "00', code: '##&quot; &quot;00&quot; &quot;00' },
    { id: 231, text: '#" "00" "00" "00', code: '#&quot; &quot;00&quot; &quot;00&quot; &quot;00' },
    { id: 232, text: '0##"/"000" "00" "00', code: '0##&quot;/&quot;000&quot; &quot;00&quot; &quot;00' },
    { id: 233, text: '[Green]0.00_ ;[Blue]\-0.00\ ', code: '[Green]0.00_ ;[Blue]\-0.00\ ' },
    { id: 234, text: '#\ ?????/?????', code: '#\ ?????/?????' },
    { id: 235, text: '[h]:mm:ss.000', code: '[h]:mm:ss.000' }
];